@@ -27,6 +27,8 @@ from gremlin_python.process.strategies import OptionsStrategy
 from gremlin_python.process.traversal import Bytecode
 import uuid
 
+log = logging.getLogger("gremlinpython")
+
 __author__ = 'David M. Brown (davebshow@gmail.com), Lyndon Bauto (lyndonb@bitquilltech.com)'
 
 
@@ -38,7 +40,7 @@ class DriverRemoteConnection(RemoteConnection):
                  message_serializer=None, graphson_reader=None,
                  graphson_writer=None, headers=None, session=None,
                  **transport_kwargs):
-        logging.info("Creating DriverRemoteConnection with url '%s'", str(url))
+        log.info("Creating DriverRemoteConnection with url '%s'", str(url))
         self.__url = url
         self.__traversal_source = traversal_source
         self.__protocol_factory = protocol_factory
@@ -54,6 +56,11 @@ class DriverRemoteConnection(RemoteConnection):
         self.__headers = headers
         self.__session = session
         self.__transport_kwargs = transport_kwargs
+
+        # keeps a list of sessions that have been spawned from this DriverRemoteConnection
+        # so that they can be closed if this parent session is closed.
+        self.__spawned_sessions = []
+
         if message_serializer is None:
             message_serializer = serializer.GraphSONMessageSerializer(
                 reader=graphson_reader,
@@ -74,11 +81,24 @@ class DriverRemoteConnection(RemoteConnection):
         self._traversal_source = self._client._traversal_source
 
     def close(self):
-        logging.info("closing DriverRemoteConnection with url '%s'", str(self._url))
+        # close this client and any DriverRemoteConnection instances spawned from this one
+        # for a session
+        if len(self.__spawned_sessions) > 0:
+            log.info("closing spawned sessions from DriverRemoteConnection with url '%s'", str(self._url))
+            for spawned_session in self.__spawned_sessions:
+                spawned_session.close()
+            self.__spawned_sessions.clear()
+
+        if self.__session:
+            log.info("closing DriverRemoteConnection with url '%s' with session '%s'",
+                         str(self._url), str(self.__session))
+        else:
+            log.info("closing DriverRemoteConnection with url '%s'", str(self._url))
+
         self._client.close()
 
     def submit(self, bytecode):
-        logging.debug("submit with bytecode '%s'", str(bytecode))
+        log.debug("submit with bytecode '%s'", str(bytecode))
         result_set = self._client.submit(bytecode, request_options=self._extract_request_options(bytecode))
         results = result_set.all().result()
         return RemoteTraversal(iter(results))
@@ -91,7 +111,7 @@ class DriverRemoteConnection(RemoteConnection):
         self.submit_async(message, bindings, request_options)
 
     def submit_async(self, bytecode):
-        logging.debug("submit_async with bytecode '%s'", str(bytecode))
+        log.debug("submit_async with bytecode '%s'", str(bytecode))
         future = Future()
         future_result_set = self._client.submit_async(bytecode, request_options=self._extract_request_options(bytecode))
 
@@ -106,14 +126,17 @@ class DriverRemoteConnection(RemoteConnection):
         future_result_set.add_done_callback(cb)
         return future
 
+    def is_closed(self):
+        return self._client.is_closed()
+
     def is_session_bound(self):
         return self.__session is not None
 
     def create_session(self):
-        logging.info("Creating session based connection")
+        log.info("Creating session based connection")
         if self.is_session_bound():
             raise Exception('Connection is already bound to a session - child sessions are not allowed')
-        return DriverRemoteConnection(self.__url,
+        conn = DriverRemoteConnection(self.__url,
                                       traversal_source=self.__traversal_source,
                                       protocol_factory=self.__protocol_factory,
                                       transport_factory=self.__transport_factory,
@@ -128,13 +151,15 @@ class DriverRemoteConnection(RemoteConnection):
                                       headers=self.__headers,
                                       session=uuid.uuid4(),
                                       **self.__transport_kwargs)
+        self.__spawned_sessions.append(conn)
+        return conn
 
     def commit(self):
-        logging.info("Submitting commit graph operation.")
+        log.info("Submitting commit graph operation.")
         return self._client.submit(Bytecode.GraphOp.commit())
 
     def rollback(self):
-        logging.info("Submitting rollback graph operation.")
+        log.info("Submitting rollback graph operation.")
         return self._client.submit(Bytecode.GraphOp.rollback())
 
     @staticmethod
